@@ -23,26 +23,26 @@
 #ifndef AD_NOTE_PARAMETERS_H
 #define AD_NOTE_PARAMETERS_H
 
-
 #include "../globals.h"
-#include "EnvelopeParams.h"
-#include "LFOParams.h"
-#include "FilterParams.h"
-#include "../Synth/OscilGen.h"
-#include "../Synth/Resonance.h"
-#include "../Misc/Util.h"
-#include "../Misc/XMLwrapper.h"
-#include "../DSP/FFTwrapper.h"
-#include "Presets.h"
+#include "PresetsArray.h"
 
-enum FMTYPE {NONE,MORPH,RING_MOD,PHASE_MOD,FREQ_MOD,PITCH_MOD};
+enum FMTYPE {
+    NONE, MORPH, RING_MOD, PHASE_MOD, FREQ_MOD, PITCH_MOD
+};
+static const int ADnote_unison_sizes[] =
+{1, 2, 3, 4, 5, 6, 8, 10, 12, 15, 20, 25, 30, 40, 50, 0};
 
 /*****************************************************************/
 /*                    GLOBAL PARAMETERS                          */
 /*****************************************************************/
 
 struct ADnoteGlobalParam {
-
+    ADnoteGlobalParam();
+    ~ADnoteGlobalParam();
+    void defaults();
+    void add2XML(XMLwrapper *xml);
+    void getfromXML(XMLwrapper *xml);
+    void paste(ADnoteGlobalParam &a);
     /* The instrument type  - MONO/STEREO
     If the mode is MONO, the panning of voices are not used
     Stereo=1, Mono=0. */
@@ -53,24 +53,24 @@ struct ADnoteGlobalParam {
     /******************************************
     *     FREQUENCY GLOBAL PARAMETERS        *
     ******************************************/
-    unsigned short int PDetune;//fine detune
-    unsigned short int PCoarseDetune;//coarse detune+octave
-    unsigned char PDetuneType;//detune type
+    unsigned short int PDetune; //fine detune
+    unsigned short int PCoarseDetune; //coarse detune+octave
+    unsigned char      PDetuneType; //detune type
 
-    unsigned char PBandwidth;//how much the relative fine detunes of the voices are changed
+    unsigned char PBandwidth;      //how much the relative fine detunes of the voices are changed
 
-    EnvelopeParams *FreqEnvelope; //Frequency Envelope
+    EnvelopeParams *FreqEnvelope;    //Frequency Envelope
 
-    LFOParams *FreqLfo;//Frequency LFO
+    LFOParams *FreqLfo; //Frequency LFO
 
     /********************************************
     *     AMPLITUDE GLOBAL PARAMETERS          *
     ********************************************/
 
     /* Panning -  0 - random
-    	      1 - left
-    	     64 - center
-    	    127 - right */
+              1 - left
+             64 - center
+            127 - right */
     unsigned char PPanning;
 
     unsigned char PVolume;
@@ -81,7 +81,8 @@ struct ADnoteGlobalParam {
 
     LFOParams *AmpLfo;
 
-    unsigned char PPunchStrength,PPunchTime,PPunchStretch,PPunchVelocitySensing;
+    unsigned char PPunchStrength, PPunchTime, PPunchStretch,
+                  PPunchVelocitySensing;
 
     /******************************************
     *        FILTER GLOBAL PARAMETERS        *
@@ -103,6 +104,8 @@ struct ADnoteGlobalParam {
 
     //how the randomness is applied to the harmonics on more voices using the same oscillator
     unsigned char Hrandgrouping;
+
+    static const rtosc::Ports &ports;
 };
 
 
@@ -111,10 +114,37 @@ struct ADnoteGlobalParam {
 /*                    VOICE PARAMETERS                     */
 /***********************************************************/
 struct ADnoteVoiceParam {
-
+    void getfromXML(XMLwrapper *xml, unsigned nvoice);
+    void add2XML(XMLwrapper *xml, bool fmoscilused);
+    void paste(ADnoteVoiceParam &p);
+    void defaults(void);
+    void enable(const SYNTH_T &synth, FFTwrapper *fft, Resonance *Reson);
+    void kill(void);
+    float getUnisonFrequencySpreadCents(void) const;
     /** If the voice is enabled */
     unsigned char Enabled;
 
+    /** How many subvoices are used in this voice */
+    unsigned char Unison_size;
+
+    /** How subvoices are spread */
+    unsigned char Unison_frequency_spread;
+
+    /** How much phase randomization */
+    unsigned char Unison_phase_randomness;
+
+    /** Stereo spread of the subvoices*/
+    unsigned char Unison_stereo_spread;
+
+    /** Vibratto of the subvoices (which makes the unison more "natural")*/
+    unsigned char Unison_vibratto;
+
+    /** Medium speed of the vibratto of the subvoices*/
+    unsigned char Unison_vibratto_speed;
+
+    /** Unison invert phase */
+    unsigned char Unison_invert_phase; //0=none,1=random,2=50%,3=33%,4=25%
+
     /** Type of the voice (0=Sound,1=Noise)*/
     unsigned char Type;
 
@@ -125,11 +155,11 @@ struct ADnoteVoiceParam {
     unsigned char Presonance;
 
     // What external oscil should I use, -1 for internal OscilSmp&FMSmp
-    short int Pextoscil,PextFMoscil;
+    short int Pextoscil, PextFMoscil;
     // it is not allowed that the externoscil,externFMoscil => current voice
 
     // oscillator phases
-    unsigned char Poscilphase,PFMoscilphase;
+    unsigned char Poscilphase, PFMoscilphase;
 
     // filter bypass
     unsigned char Pfilterbypass;
@@ -159,12 +189,12 @@ struct ADnoteVoiceParam {
     unsigned char PDetuneType;
 
     /* Frequency Envelope */
-    unsigned char PFreqEnvelopeEnabled;
+    unsigned char   PFreqEnvelopeEnabled;
     EnvelopeParams *FreqEnvelope;
 
     /* Frequency LFO */
     unsigned char PFreqLfoEnabled;
-    LFOParams *FreqLfo;
+    LFOParams    *FreqLfo;
 
 
     /***************************
@@ -172,9 +202,9 @@ struct ADnoteVoiceParam {
     ***************************/
 
     /* Panning       0 - random
-    		 1 - left
-    	        64 - center
-    	       127 - right
+             1 - left
+                64 - center
+               127 - right
        The Panning is ignored if the instrument is mono */
     unsigned char PPanning;
 
@@ -188,12 +218,12 @@ struct ADnoteVoiceParam {
     unsigned char PAmpVelocityScaleFunction;
 
     /* Amplitude Envelope */
-    unsigned char PAmpEnvelopeEnabled;
+    unsigned char   PAmpEnvelopeEnabled;
     EnvelopeParams *AmpEnvelope;
 
     /* Amplitude LFO */
     unsigned char PAmpLfoEnabled;
-    LFOParams *AmpLfo;
+    LFOParams    *AmpLfo;
 
 
 
@@ -206,12 +236,12 @@ struct ADnoteVoiceParam {
     FilterParams *VoiceFilter;
 
     /* Filter Envelope */
-    unsigned char PFilterEnvelopeEnabled;
+    unsigned char   PFilterEnvelopeEnabled;
     EnvelopeParams *FilterEnvelope;
 
     /* LFO Envelope */
     unsigned char PFilterLfoEnabled;
-    LFOParams *FilterLfo;
+    LFOParams    *FilterLfo;
 
     /****************************
     *   MODULLATOR PARAMETERS   *
@@ -247,37 +277,46 @@ struct ADnoteVoiceParam {
     unsigned char PFMDetuneType;
 
     /* Frequency Envelope of the Modullator */
-    unsigned char PFMFreqEnvelopeEnabled;
+    unsigned char   PFMFreqEnvelopeEnabled;
     EnvelopeParams *FMFreqEnvelope;
 
     /* Frequency Envelope of the Modullator */
-    unsigned char PFMAmpEnvelopeEnabled;
+    unsigned char   PFMAmpEnvelopeEnabled;
     EnvelopeParams *FMAmpEnvelope;
+
+    static const rtosc::Ports &ports;
 };
 
-class ADnoteParameters:public Presets
+class ADnoteParameters:public PresetsArray
 {
-public:
-    ADnoteParameters(FFTwrapper *fft_);
-    ~ADnoteParameters();
+    public:
+        ADnoteParameters(const SYNTH_T &synth, FFTwrapper *fft_);
+        ~ADnoteParameters();
 
-    ADnoteGlobalParam GlobalPar;
-    ADnoteVoiceParam VoicePar[NUM_VOICES];
+        ADnoteGlobalParam GlobalPar;
+        ADnoteVoiceParam  VoicePar[NUM_VOICES];
 
-    void defaults();
-    void add2XML(XMLwrapper *xml);
-    void getfromXML(XMLwrapper *xml);
+        void defaults();
+        void add2XML(XMLwrapper *xml);
+        void getfromXML(XMLwrapper *xml);
 
-    REALTYPE getBandwidthDetuneMultiplier();
-private:
-    void defaults(int n);//n is the nvoice
+        void paste(ADnoteParameters &a);
+        void pasteArray(ADnoteParameters &a, int section);
 
-    void EnableVoice(int nvoice);
-    void KillVoice(int nvoice);
-    FFTwrapper *fft;
 
-    void add2XMLsection(XMLwrapper *xml,int n);
-    void getfromXMLsection(XMLwrapper *xml,int n);
+        float getBandwidthDetuneMultiplier() const;
+        float getUnisonFrequencySpreadCents(int nvoice) const;
+        int get_unison_size_index(int nvoice) const;
+        void set_unison_size_index(int nvoice, int index);
+        static const rtosc::Ports &ports;
+        void defaults(int n); //n is the nvoice
+        void add2XMLsection(XMLwrapper *xml, int n);
+        void getfromXMLsection(XMLwrapper *xml, int n);
+    private:
+
+        void EnableVoice(const SYNTH_T &synth, int nvoice);
+        void KillVoice(int nvoice);
+        FFTwrapper *fft;
 };
 
 #endif
